// MainWindow.java

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.TableModel;
import javax.sound.midi.*;

/**
	CEChE

	@version	0.01 11/18/2000
	@author		Takahiro Shigemoto
*/
public class MainWindow extends JFrame
{
	// e萔
	private final int rowHeight = 25;	// s̍
	private final Dimension defaultWindowSize = new Dimension(600, 400);	// EChȆ傫
	private final int[] column_width = {50, 100, 60, 150, 150, 100};
	private final JLabel default_view = new JLabel("");

	// JR|[lg
	JTable table;
	JScrollPane scrollpane;
	JComboBox track_selector;

	// f[^ێpf
	TableModel[] model_array;

	public MainWindow()
	{
		// EChẼZbg
		super("Event List Viewer");
		setSize(defaultWindowSize);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// r[̍\z
		scrollpane = new JScrollPane(default_view);

		// gbNZN^̍\z
		track_selector = createTrackSelector();

		// j[̍\z
		JMenuBar menu_bar = createMenuBar();

		// EChEւ̃R|[lg̔zu
		setJMenuBar(menu_bar);
		getContentPane().add(track_selector, BorderLayout.SOUTH);
		getContentPane().add(scrollpane);
	}

	// e[ůe̕ƍs̍ݒ肷
	private void setTableWidthHeight(TableModel m)
	{
		for(int i=0; i<m.getColumnCount(); i++)
		{
			table.getColumn(m.getColumnName(i)).setPreferredWidth(column_width[i]);
		}
		table.setRowHeight(rowHeight);
	}

	// r[\z
	private Component createView(File midi_file)
	{
		try
		{
			// t@C̓ǂݍ
			Sequence sequence = MidiSystem.getSequence(midi_file);
			Track[] tracks = sequence.getTracks();

			model_array = new TableModel[tracks.length];
			for(int i=0; i<model_array.length; i++)
			{
				model_array[i] = new EventListTableModel(tracks[i], i);
			}

			// ftHǵAgbN0\
			TableModel default_model = model_array[0];
			table = new JTable(default_model);
			setTableWidthHeight(default_model);

			return(table);
		}
		catch(Exception ex)
		{
			return(null);
		}
	}

	// gbNZN^\z
	private JComboBox createTrackSelector()
	{
		JComboBox track_selector = new JComboBox();

		ItemListener track_selector_listener = new ItemListener()
		{
			public void itemStateChanged(ItemEvent ie)
			{
				if(ie.getStateChange() == ItemEvent.SELECTED)
				{
					// "Trackxx"oAxx𐔒l
					String str_track_number = (String)ie.getItem();
					int track_number = Integer.parseInt(str_track_number.substring(5, str_track_number.length()));

					table.setModel(model_array[track_number]);
					setTableWidthHeight(model_array[track_number]);
				}
			}
		};
		track_selector.addItemListener(track_selector_listener);

		return(track_selector);
	}

	// gbNZN^ɃACeݒ肷
	private void setTrackSelectorItem()
	{
		track_selector.removeAllItems();

		for(int i=0; i<model_array.length; i++)
		{
			track_selector.addItem("track" + i);
		}
	}

	// j[o[\z
	private JMenuBar createMenuBar()
	{
		JMenu file_menu = createFileMenu();

		JMenuBar menu_bar = new JMenuBar();
		menu_bar.add(file_menu);

		return(menu_bar);
	}

	// t@Cj[\z
	private JMenu createFileMenu()
	{
		JMenu file_menu = new JMenu("File");

		// File/Openj[
		JMenuItem open_file_menu = new JMenuItem("Open...");
		ActionListener open_file_menu_listener = new ActionListener()
		{
			public void actionPerformed(ActionEvent ae)
			{
				// t@CI_CAO\
				JFileChooser file_choose = new JFileChooser();
				file_choose.setFileFilter(new MIDIFileFilter());
				file_choose.setMultiSelectionEnabled(false);
				int ret = file_choose.showOpenDialog(MainWindow.this);

				// t@CIꂽꍇ́Ãt@C͂Ar[Ɍʂ\
				if(ret == JFileChooser.APPROVE_OPTION)
				{
					JViewport vp = new JViewport();
					vp.setView(createView(file_choose.getSelectedFile()));
					scrollpane.setViewport(vp);
					setTrackSelectorItem();
				}
			}
		};
		open_file_menu.addActionListener(open_file_menu_listener);

		// File/Closej[
		JMenuItem close_file_menu = new JMenuItem("Close");
		ActionListener close_file_menu_listener = new ActionListener()
		{
			public void actionPerformed(ActionEvent ae)
			{
				// ftHgԂɖ߂
				JViewport vp = new JViewport();
				vp.setView(default_view);
				scrollpane.setViewport(vp);
				track_selector.removeAllItems();
			}
		};
		close_file_menu.addActionListener(close_file_menu_listener);

		// File/Exitj[
		JMenuItem exit_file_menu = new JMenuItem("Exit");
		ActionListener exit_file_menu_listener = new ActionListener()
		{
			public void actionPerformed(ActionEvent ae)
			{
				// EChEɑ΂āAĨCxg𑗂
				MainWindow.this.dispatchEvent(new WindowEvent(MainWindow.this, WindowEvent.WINDOW_CLOSING));
			}
		};
		exit_file_menu.addActionListener(exit_file_menu_listener);

		// Filej[ɑ΂āAej[ACeǉ
		file_menu.add(open_file_menu);
		file_menu.add(close_file_menu);
		file_menu.addSeparator();
		file_menu.add(exit_file_menu);

		return(file_menu);
	}
}